<?php
//add menu in woocommerce plugin and Endpoint url

/**
 * require path
 */
require_once(GOZENGROWTH_BAE_PATH . 'page/generalpage.php');
require_once(GOZENGROWTH_BAE_PATH . 'page/loginpage.php');

class GozenApi
{

    public function initAction()
    {
        add_action('admin_menu', array(&$this, 'addmenu'));
        add_action('rest_api_init', array(&$this, 'registerEndpoint'));
    }

    //add menu in woocommerce 
    public function addmenu()
    {
        if (Pluginis_active()) {
            add_submenu_page('woocommerce', 'Gozen', 'GoZen', 'manage_woocommerce', 'gozen', array(&$this, 'init_gozen_page'));
            //register the script dir and style.
            wp_register_script('gozen-script', GOZENGROWTH_BAE_URL . 'java_script/gozen.js', array('jquery'), GOZENGROWTH_VERSION);
            wp_register_style('gozen-style', GOZENGROWTH_BAE_URL . 'css/gozen.css', array(), GOZENGROWTH_VERSION);
        } else {
            deactivate_plugins(GOZENGROWTH_FILE_NAME);
            add_action('admin_notices', 'gozengrowthrequied');
        }
    }
    //Endpoint url
    public function registerEndpoint()
    {

        register_rest_route('gozen-growth/v1', 'keySecret', [
            'methods' => 'POST',
            'callback' => array($this, 'keySecret'),
            'permission_callback' => '__return_true',
        ]);

        register_rest_route("gozen-growth/v1", "dirto_Logout", [
            'methods' => 'POST',
            'callback' => array($this, 'dirto_Logout'),
            'permission_callback' => '__return_true',
        ]);
        register_rest_route("gozen-growth/v1", "syncUser", [
            'methods' => 'POST',
            'callback' => array($this, 'syncUser'),
            'permission_callback' => '__return_true',
        ]);
    }

    /**
     * If woocommerce plugin is active, add the gozen submenu into woocommerce menu.
     */
    public function init_gozen_page()
    {


        /**
         * add the script file and style file in wp.
         */
        wp_enqueue_script('gozen-script');
        wp_enqueue_style('gozen-style');

        //decrypt the GOZEN_API_KEY if the APi_key is not empty.
        $api = GOZENGROWTH_API_KEY ? apiSecret(GOZENGROWTH_API_KEY, "d") : " ";
        $s_id = GOZENGROWTH_SHOP_UUID;

        $add_nonce = wp_create_nonce("test-plugin-js");
        $ajax_url = admin_url('admin-ajax.php');
        $result_server_url = GOZENGROWTH_SERVER_URL;

        $result = [
            'server_url' => $result_server_url,
            'api_key' => $api,
            'shop_uuid' => $s_id,
            'status' => GOZENGROWTH_LOGIN_STATUS,
            'sync_user' => GOZENGROWTH_SYNC_USER

        ];

        wp_localize_script('gozen-script', 'gozen_test_url', array('ajax_url' => $ajax_url, 'nonce' => $add_nonce, 'data' => $result));

        /**
         *if the Login status is true move into the main_container page,elae it turn into Login page.
         */
        if (GOZENGROWTH_LOGIN_STATUS == 1) {
            gozen_mainPage();
        } else {
            gozen_loginPage();
        }
    }
    /**
     * Update the consumer key & secret of user.
     * @param $response_data
     * @return mixed
     */

    public function keySecret($response_data)
    {
        global $wpdb;

        //get the response datea form body and decode the data
        $data = $response_data->get_body();
        $data = json_decode($data, true);
        print_r($data);

        //store the 
        $consumer_key = $data["consumer_key"];
        $consumer_secret = $data["consumer_secret"];
        $user_id = $data["user_id"];
        $key_permission = $data["key_permissions"];

        $c_key = apiSecret($consumer_key, "e");
        $c_secret = apiSecret($consumer_secret, "e");


        //GET user from DB
        // $user=$wpdb->get_row($wpdb->prepare("SELECT * FROM $wpdb->users WHERE ID=%d",$user_id));
        // $username=$user->user_nicename;
        // $useremail=$user->user_email;

        $inset = $wpdb->update($wpdb->prefix . 'gozen_growth', array(
            "user_id" => $user_id,
            // "username"=>$username,
            // "user_email"=>$useremail,
            "consumer_key" => $c_key,
            "consumer_secret" => $c_secret,
            "key_permission" => $key_permission,
            "wc_auth_approve_status" => 1
        ), array("key_id" => 1));

        //send consumer_key,consumer_secret, to gozen
        $w_url = GOZENGROWTH_USER_HOST_URL;
        //set the post request url.
        $url = GOZENGROWTH_SERVER_URL . '/api/v1/public/woocommerce/install';

        $body_content = array(
            "consumer_key" => $consumer_key,
            "consumer_secret" => $consumer_secret,
            "woocommerce_url" => $w_url
        );

        //get cuurent api_key form DB.
        $apikey = $wpdb->get_col($wpdb->prepare("SELECT api_key FROM {$wpdb->prefix}gozen_growth"));
        $apikey = apiSecret($apikey[0], 'd');


        $message = '';
        $status = 0;
        $data = [];
        $res = [
            'message' => $message,
            'status' => $status,
            'data' => $data
        ];

        //encode the $body_content value.
        $body_cont = json_encode($body_content, true);


        $option = [
            'body' => $body_cont,
            'headers' => [
                'content-type' => 'application/json',
                'api-key' => $apikey
            ],
            'data_format' => 'body'
        ];

        //send the post request with data to gozen
        $res = wp_remote_post($url, $option);

        //validate the reaponse 
        if (is_array($res) && is_wp_error($res)) {
            echo "<script>alert('error');</script>";
        } else {

            //get the shop_uuid form response and store the data in DB.
            $bodyData = json_decode($res['body'], true);
            $body = $bodyData['shop_uuid'];

            $insert = $wpdb->update($wpdb->prefix . 'gozen_growth', array(
                "shop_uuid" => $body,
                "Login_status" => 1
            ), array("key_id" => 1));
        }

        $response = array('success' => true, 'RESPONSE_CODE' => 'PROCESSED');
        $status = 200;


        return new \WP_REST_Response($response, $status);
    }

    /**
     * Endpoint callback function for dirto_Logout,it update the login status
     * @return mixed
     */
    public function dirto_Logout()
    {

        global $wpdb;

        $wpdb->update($wpdb->prefix . 'gozen_growth', array('Login_status' => 0), array("key_id" => 1));

        $response = array('success' => true, 'RESPONSE_CODE' => 'PROCESSED');
        $status = 200;

        return new \WP_REST_Response($response, $status);
    }

    /**
     * update the sync_user response data 
     * @return mixed
     */
    public function syncUser($response)
    {
        global $wpdb;

        $data = $response->get_body();
        $data = json_decode($data, true);

        if ($data["records"] !== 0) {
            $wpdb->update($wpdb->prefix . 'gozen_growth', array('sync_user' => $data["records"]), array("key_id" => 1));
        }

        $response = array('success' => true, 'RESPONSE_CODE' => 'PROCESSED');
        $status = 200;

        return new \WP_REST_Response($response, $status);
    }
}
function gozengrowthrequied()
{

?>

<div id="message" class="notice notice-error">
    <h1 id="errorheader">GoZen <span id="title_content">Growth</span></h1>
    <P>GoZen Growth requires WooCommerce to work seamlessly. Please install!</p>
</div>

<?php
}
?>