<?php

class Gozen_Growth_Utils
{
    /**
     * validate hmac and verify authenticity of request
     * @param  array $request_body request body
     * @return boolean
     */

    public static function is_hmac_valid($request)
    {

        $hmac = $request->get_header('X-Gozen-Hmac');

        $body = $request->get_body();

        $secret = Gozen_Growth_Utils::get_gozen_secret();

        $hash = base64_encode(hash_hmac('sha256', $body, $secret, true));

        if ($hash === $hmac) return true;

        return false;
    }

    /**
     * sanitize request body
     * @param  Object $request pass whole request
     * @return array  sanitized request body
     */
    public static function sanitized_body($request)
    {

        $body = $request->get_body();
        $body = json_decode($body, true);
        return $body;
    }
    /**
     * sanitize query parameters
     * @param  Object $request pass whole request
     * @return array  sanitized query parameters
     */

    public static function sanitized_params($request)
    {

        $params = $request->get_params();

        foreach ($params as $key => $value) {
            $params[$key] = sanitize_text_field($value);
        }

        return $params;
    }

    /**
     * get user who activated  plugin
     * 
     * @return int user id
	 
     */
    public static function get_plugin_user_id()
    {

        return get_option(GOZEN_GROWTH_USER, '');
    }

    /**
     * set user who activated  plugin
     * @return int user id
     */

    public static function set_plugin_user_id($id)
    {

        return update_option(GOZEN_GROWTH_USER, $id);
    }
    /**
     * remove user who activated  plugin
     * @return int user id
     */

    public static function remove_plugin_user_id()
    {
        return delete_option(GOZEN_GROWTH_USER);
    }


    /**
     * get gozen access token
     */
    public static function get_gozen_secret()
    {

        return get_option(GOZEN_SECRET, '');
    }

    /**
     * set gozen access_token | gozen secret
     */
    public static function set_gozen_secret($secret)
    {
        update_option(GOZEN_SECRET, $secret);
    }

    /**
     * generate hmac for request
     * @param  array $request pass whole request
     * @return array hmac included request array
     */
    public static function generate_hmac($body)
    {

        $secret = get_option(GOZEN_SECRET, '');

        $querystring = http_build_query($body);

        $hash = hash_hmac('sha256', $querystring, $secret);

        $new_body = array_merge($body, array('hmac' => $hash));

        return $new_body;
    }

    public static function validate_code_with_growth_server($code)
    {
        $url = Gozen_Env::get_gozen_url();

        $url .= '/api/v1/integration-v2/woocommerce/auth/grand';

        $response = wp_remote_post(
            $url,
            array(
                'headers'     => array('Content-Type' => 'application/json; charset=utf-8'),

                'body'        => json_encode(array(
                    'grant_type' => 'authorization_code',
                    'code' => $code,
                ),),

                'method'      => 'POST',

                'data_format' => 'body',

                'timeout' => 15,

                'sslverify' => false,
            )
        );

        return $response;
    }


    public static function dnd($out)
    {
        echo "<pre>\r\n";
        var_dump($out);
        echo "\r\n</pre>";
        die();
    }

    public static function send_response($raw_response)
    {
        $response_array = Gozen_Growth_Utils::generate_hmac($raw_response);

        $response = new WP_REST_Response();

        $response->set_status(200);

        $response->set_data($response_array);

        $response = rest_ensure_response($response);

        return $response;
    }
}