<?php
class Gozen_Growth_Webhook
{

    public static  function createWebhook($userID, $name, $topic, $secret, $api_version, $deliveryURL, $status)
    {
        if (Gozen_Growth_Webhook::is_webhook_exist($topic, $deliveryURL)) {

            return new WP_Error("Conflict", "webhook with $deliveryURL and $topic already exist", array('status' => 409));
        }

        $webhook = new WC_Webhook();

        $webhook->set_user_id($userID); // User ID used while generating the webhook payload.

        $webhook->set_name($name); // name of webhook

        $webhook->set_topic($topic); // Event used to trigger a webhook.

        $webhook->set_secret($secret); // Secret to validate webhook when received.

        $webhook->set_api_version($api_version); // API version used to generate the webhook payload.

        $webhook->set_delivery_url($deliveryURL); // URL where webhook should be sent.

        $webhook->set_status($status); // Webhook status.

        $save = $webhook->save();

        return $save;
    }

    public static function delete_webhook($webhook_id)
    {

        $webhook = new WC_Webhook();

        $webhook->set_id($webhook_id);

        if ($webhook->delete()) return $webhook_id;

        return false;
    }

    public static function delete_all_webhook($domain)
    {

        $hooks =  Gozen_Growth_Webhook::get_all_webhook($domain);

        if (empty($hooks)) return new WP_REST_Response(array('status' => 'success'), 200);

        foreach ($hooks as $hook) {

            Gozen_Growth_Webhook::delete_webhook($hook->webhook_id);
        }

        return new WP_REST_Response(array('status' => 'success'), 200);
    }

    public static function get_webhook($webhook_id)
    {

        $webhook = new WC_Webhook();

        $webhook->set_id($webhook_id);

        $webhook_data = $webhook->get_data();

        return $webhook_data;
    }

    public static function get_all_webhook($delivery_domain)
    {

        global $wpdb;

        $query = "select * from {$wpdb->prefix}wc_webhooks where delivery_url like '$delivery_domain%'";

        $hooks = $wpdb->get_results($query, OBJECT);

        return $hooks;
    }

    public static function is_webhook_exist($topic, $address)
    {
        global $wpdb;

        $query = "select * from {$wpdb->prefix}wc_webhooks where delivery_url = '$address' and topic = '$topic'";

        $hooks = $wpdb->get_results($query, OBJECT);

        return count($hooks) !== 0;
    }
}