<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @since      1.0.0
 *
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 */

class Gozn_Groth_Public
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}



	/**
	 * Register the REST API routes for the plugin.
	 *
	 * @since    1.0.0
	 */

	public function add_api_routes()
	{

		register_rest_route('gozen/v1', '/webhook/', array(
			'methods' => WP_REST_Server::CREATABLE,
			'callback' => array($this, 'create_webhook'),
			'permission_callback' => array($this, 'authenticate')
		));

		register_rest_route('gozen/v1', '/webhook/', array(
			'methods' => WP_REST_Server::READABLE,
			'callback' => array($this, 'get_all_webhooks'),
			'permission_callback' => array($this, 'authenticate')
		));

		register_rest_route('gozen/v1', '/webhook/', array(
			'methods' => WP_REST_Server::DELETABLE,
			'callback' => array($this, 'delete_all_webhooks'),
			'permission_callback' => array($this, 'authenticate')
		));

		register_rest_route('gozen/v1', '/webhook/(?P<id>\d+)', array(
			'methods' => WP_REST_Server::DELETABLE,
			'callback' => array($this, 'delete_webhook'),
			'permission_callback' => array($this, 'authenticate')
		));

		register_rest_route('gozen/v1', '/connect/', array(
			'methods' => WP_REST_Server::CREATABLE,
			'callback' => array($this, 'connect_shop_request_from_growth'),
			'permission_callback' => function () {
				return true;
			},
		));
	}


	public function write_log($log)
	{
		if (is_array($log) || is_object($log)) {
			error_log(print_r($log, true));
		} else {
			error_log($log);
		}
	}


	public function authenticate($request)
	{
		return true;

		if (Gozen_Growth_Utils::is_hmac_valid($request)) return true;

		return false;
	}

	public function connect_shop_request_from_growth($request)
	{
		try {



			$params = Gozen_Growth_Utils::sanitized_params($request);

			if (!isset($params['code'])) return new WP_Error("Unauthorized_access", "Unable to verify request originated from gozen growth", array('status' => 401));


			$code = $params['code'];

			//validate code recieved with growth server and complete handshake


			$validation_response = Gozen_Growth_Utils::validate_code_with_growth_server($code);




			$response_from_gozen = $validation_response;

			if ($response_from_gozen["response"]["code"] !== 200) {

				return new WP_Error("Unauthorized_access", "Unable to verify request originated from gozen growth", array('status' => 401));
			}




			$gozen_response_body = json_decode($response_from_gozen["body"]);




			if (!isset($gozen_response_body->access_token)) {

				return new WP_Error("Unauthorized_access", "Unable to verify request originated from gozen growth", array('status' => 401));
			}

			// validation completed now return access token to growth server and update in db

			$access_token = $gozen_response_body->access_token;




			Gozen_Growth_Utils::set_gozen_secret($access_token);


			$response = new WP_REST_Response();

			$response->set_data($gozen_response_body);

			$response = rest_ensure_response($response);

			return $response;
		} catch (Exception $err) {
			return new WP_Error("unable to handle request", "Unable to handle request", array('status' => 401));
		}
	}

	/**
	 * Create a webhook
	 *@param WP_REST_Request $request
	 * @since    1.0.0
	 */
	public function create_webhook($request)
	{

		$required_params = array('name', 'topic', 'hook');

		$body = $request->get_json_params();

		foreach ($required_params as $parameter) {

			if (!isset($body[$parameter])) {

				return new WP_Error("invalid_parameter", "Invalid parameter $parameter", array('status' => 400));
			}
		}

		$sanitized_body = Gozen_Growth_Utils::sanitized_body($request);

		$user = Gozen_Growth_Utils::get_plugin_user_id();

		$title = $sanitized_body['name'];

		$topic = $sanitized_body['topic'];

		$hook = $sanitized_body['hook'];

		$status = 'active';

		$api_version = 3;

		$secret = Gozen_Growth_Utils::get_gozen_secret();

		$new_webhook_id = Gozen_Growth_Webhook::createWebhook($user, $title, $topic, $secret, $api_version, $hook, $status);

		if (is_wp_error($new_webhook_id)) return $new_webhook_id;

		$response_array = array(

			"webhook_id" => $new_webhook_id,
			'name' => $title,
			'topic' => $topic,
			'hook' => $hook,
			'status' => $status,
			'api_version' => $api_version,

		);

		$response_array = Gozen_Growth_Utils::generate_hmac($response_array);

		$response = new WP_REST_Response();

		$response->set_status(200);

		$response->set_data($response_array);

		$response = rest_ensure_response($response);

		return $response;
	}

	function delete_webhook($request)
	{
		$webhook =  $request['id'];

		if (Gozen_Growth_Webhook::delete_webhook($webhook)) {

			return new WP_REST_Response(array('status' => 'success'), 200);
		} else {

			return new WP_REST_Response(array('status' => 'failure'), 400);
		}
	}

	public function get_all_webhooks($request)
	{
		$domain =  Gozen_Growth_Utils::sanitized_body($request)['domain'];

		if (empty($domain)) return new WP_Error("invalid_parameter", "Invalid parameter domain", array('status' => 400));

		return Gozen_Growth_Webhook::get_all_webhook($domain);
	}

	public function delete_all_webhooks($request)
	{
		$domain =  Gozen_Growth_Utils::sanitized_body($request)['domain'];

		if (empty($domain)) return new WP_Error("invalid_parameter", "Invalid parameter domain", array('status' => 400));

		return Gozen_Growth_Webhook::delete_all_webhook($domain);
	}

	/**
	 * this function will add new webhook topic 
	 */
	public function add_new_topic_hooks($topic_hooks)
	{

		$new_hooks = array(
			'action.wc_gozen_cart_updated' => array(
				'wc_gozen_cart_updated',
			),
			'action.wc_gozen_checkout_updated' => array(
				'wc_gozen_checkout_updated',
			),
		);

		return array_merge($topic_hooks, $new_hooks);
	}

	public function add_new_topic_events($topic_events)
	{
		// New events to be used for resources.
		$new_events = array(
			'wc_gozen_cart_updated',
			'wc_gozen_checkout_updated',
		);

		return array_merge($topic_events, $new_events);
	}


	public function format_products()
	{
		$line_items = array();
		foreach (WC()->cart->get_cart() as $cart_item_key => $cart_item) {



			$product = array();
			$product['key'] = $cart_item['key'];
			$product['id'] = $cart_item['product_id'];
			$product['quantity'] = $cart_item['quantity'];
			$product['data_hash'] = $cart_item['data_hash'];


			$wc_product = new WC_Product($product['id']);

			$product['name'] = $wc_product->get_name();
			$product['short_description'] = $wc_product->get_short_description();
			$product['description'] = $wc_product->get_description();
			$product['sku'] = $wc_product->get_sku();
			$product['backorders_allowed'] = $wc_product->backorders_allowed();
			$product['sold_individually'] = $wc_product->get_sold_individually();
			$product['permalink'] = $wc_product->get_permalink();
			$product['totals'] = array(
				"line_subtotal"		=>	$cart_item['line_subtotal'],
				"line_subtotal_tax"		=>	$cart_item['line_subtotal_tax'],
				"line_total"		=>	$cart_item['line_total'],
				"line_total_tax"		=>	$cart_item['line_tax'],

			);
			$product['prices'] = array(
				"price"		=>	$wc_product->get_price(),
				"regular_price"		=>	$wc_product->get_regular_price(),
				"sale_price"		=>	$wc_product->get_sale_price(),
				"price_range"		=>	$wc_product->get_sale_price(),
			);
			$product['variation'] = $wc_product->get_children();
			$product['item_data'] = $wc_product->get_data();

			$line_items[] = $product;
		}
		return $line_items;
	}

	public function itrate_meta_data_array($metadatas)
	{
		$result = array();
		foreach ($metadatas as $metadata) {
			$result[] = $metadata->get_data();
		}
		return $result;
	}

	/**
	 * this function will run when an item is added to cart.
	 * */

	public function cart_updated_action()
	{
		try {

			$current_user = wp_get_current_user();


			if (!isset(WC()->cart)) return;
			if (!isset($current_user->user_email) || empty($current_user->user_email)) return;

			$cart = array();
			$customer = array();
			$user_id = $current_user->ID;
			$current_user = new WC_Customer($user_id);



			// cart data

			$cart['hash'] = WC()->cart->get_cart_hash();
			$cart['coupons'] = array_map(function ($coupon) {
				return $coupon->get_data();
			}, WC()->cart->get_coupons());
			$cart['shipping_address'] = WC()->cart->get_customer()->get_shipping();
			$cart['billing_address'] = WC()->cart->get_customer()->get_billing();
			$cart['items_count'] = WC()->cart->get_cart_contents_count();
			$cart['items_weight'] = WC()->cart->get_cart_contents_weight();
			$cart['needs_payment'] = WC()->cart->needs_payment();
			$cart['needs_shipping'] = WC()->cart->needs_shipping();
			$cart['has_calculated_shipping'] = WC()->cart->get_customer()->has_calculated_shipping();
			$cart['fees'] = WC()->cart->get_fees();
			$cart['totals'] = WC()->cart->get_totals();
			$cart['applied_coupons_codes'] = WC()->cart->get_applied_coupons();
			$cart['checkout_url'] = wc_get_checkout_url();
			$cart['cart_url'] = wc_get_cart_url();
			$cart['items'] = $this->format_products();
			$cart['cart'] = WC()->cart->get_cart();

			// $cart['removed_items'] = WC()->cart->get_removed_cart_contents();
			// $cart['shipping_rates'] = array_map(function ($shippingObj) {
			// 	$this->write_log($shippingObj);
			// }, WC()->cart->calculate_shipping());



			// customer data
			$customer['id'] = $user_id;
			$customer['email'] = $current_user->get_email();
			$customer['first_name'] = $current_user->get_first_name();
			$customer['last_name'] = $current_user->get_last_name();
			$customer['username'] = $current_user->get_username();
			$customer['display_name'] = $current_user->get_display_name();
			$customer['role'] = $current_user->get_role();
			$customer['date_created'] = $current_user->get_date_created()->date_i18n('Y-m-d\TH:i:s');
			$customer['date_modified'] = !empty($current_user->get_date_modified()) ? $current_user->get_date_modified()->date_i18n('Y-m-d\TH:i:s') : '';
			$customer['billing'] = $current_user->get_billing();
			$customer['shipping'] = $current_user->get_shipping();
			$customer['is_paying_customer'] = $current_user->is_paying_customer();
			$customer['avatar_url'] = $current_user->get_avatar_url();
			$customer['meta_data'] = $this->itrate_meta_data_array($current_user->get_meta_data());

			$gozen_action = current_action() === 'woocommerce_checkout_after_customer_details'
				? 'checkout_updated'
				: "cart_updated";


			$payload = array(
				"current_action"	=> 		current_action(),
				"gozen_action"		=>		$gozen_action,
				"customer"			=> 		$customer,
				"cart" 				=>		$cart,
			);

			$this->write_log(
				json_encode($payload, JSON_PRETTY_PRINT)
			);




			switch (current_action()) {

				case 'woocommerce_checkout_after_customer_details':
					do_action('wc_gozen_checkout_updated', $payload);
					break;

				default:
					do_action('wc_gozen_cart_updated', $payload);
					break;
			}
		} catch (Exception $e) {
			// Discard the error
		}
	}
}